﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;
using System.Windows.Threading;
using System.Collections;

namespace MDPoverUA
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    { 
        #region class variables
        //XXX
        // declare UaLogic class
        UaLogic logic;

        // declare delegates for cross threading operations
        Delegate enableDelegate;
        Delegate notificationDelegate;

        System.Threading.Thread mThrdMainForm = null;
        String errOutputCpuTemperature = "";

        //XXX KEINE variablen
        delegate void itemNotificationDelegate(Opc.Ua.Client.MonitoredItem monitoredItem, Opc.Ua.Client.MonitoredItemNotificationEventArgs e);
        delegate void setPageIsEnabledDelegate(bool value);

        bool flagClose = false;

        // declare varaibles for modules checking 
        bool cpuTempTested = false;
        bool cpuFreqTested = false;
        bool cpuUsageTested = false;
        bool freeSpaceTested = false;
        bool cfCardTested = false;
        bool twinCATTested = false;        

        ArrayList errorList = new ArrayList();
        #endregion

        #region constructor
        // constructor
        public Window1()
        {
            try
            {

            // XXX Copy of current instance
            // copy a instance of the current thread for cross threading operations
            mThrdMainForm = System.Threading.Thread.CurrentThread;

            InitializeComponent();

            // XXX
            // set a CPU temperature for min/max in the label
            tbCpuTempMin.Text = "20 °C";
            tbCpuTempMax.Text = "50 °C";

            // create delegates for cross threading operations
            enableDelegate = new setPageIsEnabledDelegate(this.setPageIsEnabled);
            notificationDelegate = new itemNotificationDelegate(this.item_Notification);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), ex.Message);
                throw;
            }
        }
        #endregion

        #region events
        // loaded event handler
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // XXX
            // set Statusinfo in TextBlock
            tbStatusOutput.Text = "CONNECTING";
            
            try
            {
                // try to create a logic instance
                string path = "settings.xml";
          //      logic = new UaLogic(notificationDelegate, enableDelegate, System.Threading.Thread.GetDomain().BaseDirectory.ToString() + "settings.xml", "Current Measurement");
                logic = new UaLogic(notificationDelegate, enableDelegate, path, "Current Measurement");
            }
            catch (Exception ex)
            {
                // disable controls in case of an exception
                enableDelegate.DynamicInvoke(false);
                // XXX in die logausgabe schreiben
                System.Diagnostics.Trace.Write(ex.ToString());
                MessageBox.Show(ex.ToString(), ex.Message);
            }
        }

        // button click event handler
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            // get the temperature limits
            // XXX keine geschachtelten Funktionsaufrufe
            // XXX txtCpuTempMin = txtCpuTempMin.Text.Trim

            int minTemp = System.Convert.ToInt32(tbCpuTempMin.Text.Remove(tbCpuTempMin.Text.IndexOf(' ')));
            int maxTemp = System.Convert.ToInt32(tbCpuTempMax.Text.Remove(tbCpuTempMax.Text.IndexOf(' ')));

            switch (((Button)sender).Name)
            {
                // closing the windows
                case "butClose":
                    this.Close();
                    break;

                // CPU min temperature up (maximize)
                case "butCpuTempMinUp":
                    if ((minTemp + 1) < maxTemp)
                    { tbCpuTempMin.Text = (minTemp + 1).ToString() + " °C"; }
                    // checkCpuTemperature(iTemp);
                    break;

                // CPU min temperature down (minimize)
                case "butCpuTempMinDown":
                    if ((minTemp - 1) >= 0)
                    { tbCpuTempMin.Text = (minTemp - 1).ToString() + " °C"; }
                    //checkCpuTemperature(iTemp);
                    break;

                // CPU max temperature up (maximize)
                case "butCpuTempMaxUp":
                    tbCpuTempMax.Text = (maxTemp + 1).ToString() + " °C";
                    //checkCpuTemperature(iTemp);
                    break;

                // CPU max temperature down (minimize)
                case "butCpuTempMaxDown":
                    if ((maxTemp - 1) > minTemp)
                    { tbCpuTempMax.Text = (maxTemp - 1).ToString() + " °C"; }
                    //checkCpuTemperature(iTemp);
                    break;
            }

            // clear the °C
            string strTemp = tbCpuTemp.Text.Replace(" °C", "");
            int iTemp;
            if (int.TryParse(strTemp, out iTemp))
            {
                checkCpuTemperature(iTemp);
            }
            else
            {
                // Close Button / the button is top left - the button is invisible 
                if (((Button)sender).Name == "butClose")
                {
                    // close the form and terminate application
                    this.Close();
                }
            }
        }   

        // monitored item notification event handler
        public void item_Notification(Opc.Ua.Client.MonitoredItem monitoredItem, Opc.Ua.Client.MonitoredItemNotificationEventArgs e)
        {
            try
            {
                if (!flagClose)
                {
                    // check if there could be a cross threading exception
                    if (System.Threading.Thread.CurrentThread != mThrdMainForm)
                    {
                        try
                        {
                            // prepare an invocation and set the current thread
                            Dispatcher disp = System.Windows.Threading.Dispatcher.FromThread(mThrdMainForm);
                            itemNotificationDelegate dlg = new itemNotificationDelegate(item_Notification);
                            // invoke this event
                            disp.Invoke(DispatcherPriority.Normal, dlg, monitoredItem, new object[] { e });
                        }
                        catch (Exception ex) { System.Diagnostics.Trace.Write(ex.ToString()); }
                    }
                    // if there is no risk of a cross threading exception, analyse the results
                    else
                    {
                        try
                        {
                            if (logic == null)
                                return;

                            // get notification information
                            Opc.Ua.MonitoredItemNotification notification = (Opc.Ua.MonitoredItemNotification)e.NotificationValue;

                            if (notification == null || notification.Value == null || notification.Value.WrappedValue.Value == null)
                                return;

                            object value = notification.Value.WrappedValue.Value;

                            // Read module information
                            switch (monitoredItem.DisplayName)
                            {
                                // get CPU Frequency information
                                case "Modules|CPU|CPU0|0x8nn1|0x01":
                                    tbCpuFrequency.Text = value.ToString();
                                    recDisable_CPUFreq.Visibility = Visibility.Collapsed;
                                    tbDevice.Text = " MHz";
                                    break;

                                // get CPU Usage information 
                                case "Modules|CPU|CPU0|0x8nn1|0x02":
                                    tbCpuUsage.Text = value.ToString();
                                    recDisable_CPUUsage.Visibility = Visibility.Collapsed;
                                    tbDeviceUsage.Text = " %";
                                    break;

                                // get Memory Available information
                                case "Modules|Memory|Memory|0x8nn1|0x02":
                                    int value1 = Convert.ToInt32(value) / 1024 / 1024;
                                    tbFlashCardMemory.Text = value1.ToString();
                                    recDisable_FreeSpace.Visibility = Visibility.Collapsed;
                                    tbDeviceMemory.Text = " MB";
                                    break;

                                // get SiliconDrive Drive Usage information
                                case "Modules|SiliconDrive|SiliconDrive0|0x8nn1|0x02":
                                    tbUsageCard.Text = value.ToString();
                                    recDisable_CFCard.Visibility = Visibility.Collapsed;
                                    tbDeviceCard.Text = " %";
                                    break;

                                // get TwinCAT Major information
                                case "Modules|TwinCAT|TwinCAT|0x8nn1|0x01":
                                    tbMajorVersion.Text = value.ToString() + ".";
                                    recDisable_TwinCAT.Visibility = Visibility.Collapsed;
                                    break;

                                // get TwinCAT Minor information
                                case "Modules|TwinCAT|TwinCAT|0x8nn1|0x02":
                                    tbMinorVersion.Text = value.ToString();
                                    break;

                                // get TwinCAT build information
                                case "Modules|TwinCAT|TwinCAT|0x8nn1|0x03":
                                    tbBuild.Text = "  Build " + value.ToString();
                                    recDisable_TwinCAT.Visibility = Visibility.Collapsed;
                                    break;

                                // get CPU temperature information
                                case "Modules|CPU|CPU0|0x8nn1|0x03":
                                    tbCpuTemp.Text = value.ToString() + " °C";
                                    errOutputCpuTemperature = "";

                                    int temp;
                                    if (int.TryParse(value.ToString(), out temp))
                                    { checkCpuTemperature(temp); }

                                    break;
                            }
                        }
                        catch (Exception ex)
                        {
                            enableDelegate.DynamicInvoke(false);
                            System.Diagnostics.Trace.Write(ex.ToString());
                            return;
                        }
                    }
                }
            }
            catch (Exception ex)
            { MessageBox.Show(ex.Message); }
        }
        #endregion

        #region helper methods

        // Checked the module of exists
        void checkedModule()
        {
            // Checked the CPU temperature module 
            if (!cpuTempTested)
            {
                cpuTempTested = true;
                try
                {
                    Opc.Ua.DataValue cpuTempValue = logic.ReadValue(logic.GetNodeId("Modules|CPU|CPU0|0x8nn1|0x03"));
                }
                catch (Exception ex)
                {
                    // if the CPU temperature not exists get visible the recDisable_CPUTemp
                    recDisable_CPUTemp.Visibility = Visibility.Visible;
                    errorList.Add(" CPU temperature");
                    moduleError();
                    System.Diagnostics.Trace.Write(ex.ToString());
                    return;
                }
            }
            // Checked the CPU frequency module 
            if (!cpuFreqTested)
            {
                cpuFreqTested = true;

                try
                {
                    Opc.Ua.DataValue cpuFreqValue = logic.ReadValue(logic.GetNodeId("Modules|CPU|CPU0|0x8nn1|0x01"));
                }
                catch (Exception ex)
                {
                    // if the CPU frequency not exists get visible the recDisable_CPUFreq
                    recDisable_CPUFreq.Visibility = Visibility.Visible;
                    errorList.Add(" CPU frequency");
                    moduleError();
                    System.Diagnostics.Trace.Write(ex.ToString());
                    return;
                }
            }
            // Checked the CPU usage module 
            if (!cpuUsageTested)
            {
                cpuUsageTested = true;

                try
                {
                    Opc.Ua.DataValue cpuUsageValue = logic.ReadValue(logic.GetNodeId("Modules|CPU|CPU0|0x8nn1|0x02"));
                }
                catch (Exception ex)
                {
                    // if the CPU usage not exists get visible the recDisable_CPUUsage
                    recDisable_CPUUsage.Visibility = Visibility.Visible;
                    errorList.Add(" CPU usage");
                    moduleError();
                    System.Diagnostics.Trace.Write(ex.ToString());
                    return;
                }
            }

            // Checked the FreeSpace module
            if (!freeSpaceTested)
            {
                freeSpaceTested = true;

                try
                {
                    Opc.Ua.DataValue freeSpaceValue = logic.ReadValue(logic.GetNodeId("Modules|Memory|Memory|0x8nn1|0x02"));
                }
                catch (Exception ex)
                {
                    // if the CPU usage not exists get visible the recDisable_FreeSpace
                    recDisable_FreeSpace.Visibility = Visibility.Visible;
                    errorList.Add(" Memory");
                    moduleError();
                    System.Diagnostics.Trace.Write(ex.ToString());
                    return;
                }
            }

            // Checked the TwinCAT module
            if (!twinCATTested)
            {
                twinCATTested = true;

                try
                {
                    Opc.Ua.DataValue twinCATMajor = logic.ReadValue(logic.GetNodeId("Modules|TwinCAT|TwinCAT|0x8nn1|0x01"));
                    Opc.Ua.DataValue twinCATMinor = logic.ReadValue(logic.GetNodeId("Modules|TwinCAT|TwinCAT|0x8nn1|0x02"));
                    Opc.Ua.DataValue twinCATBuild = logic.ReadValue(logic.GetNodeId("Modules|TwinCAT|TwinCAT|0x8nn1|0x03"));
                }
                catch (Exception ex)
                {
                    // if the TwinCAT not exists get visible the recDisable_TwinCAT
                    recDisable_TwinCAT.Visibility = Visibility.Visible;
                    errorList.Add(" TwinCAT");
                    moduleError();
                    System.Diagnostics.Trace.Write(ex.ToString());
                    return;
                }
            }

            // Checked the SiliconDrive module 
            if (!cfCardTested)
            {
                cfCardTested = true;

                try
                {
                    Opc.Ua.DataValue cfCardValue = logic.ReadValue(logic.GetNodeId("Modules|SiliconDrive|SiliconDrive0|0x8nn1|0x02"));
                }
                catch (Exception ex)
                {
                    // if the CFCard not exists get visible the recDisable_CFCard
                    recDisable_CFCard.Visibility = Visibility.Visible;
                    errorList.Add(" SiliconDrive");
                    moduleError();
                    System.Diagnostics.Trace.Write(ex.ToString());
                    return;
                }
            }
        }

        // print all module errors in label
        void moduleError()
        {
            string output = string.Empty;

            for (int i = 0; i < errorList.Count; i++)
            {
                output += errorList[i] + " module";

                if (!((i+1) == errorList.Count))
                {
                    output += ", ";
                }
            }

            if (errorList.Count > 1)
            {
                tbErrorOutput_Status.Text = output + " aren't available";
            }
            else
            {
                tbErrorOutput_Status.Text = output + " isn't available";
            }
        }

        // checks the CPU temperature
        void checkCpuTemperature(int temp)
        {
            try
            {
                // if the CPU temperature to low show a warning
                errOutputCpuTemperature = "";
                string sTempMin = tbCpuTempMin.Text.Remove(tbCpuTempMin.Text.IndexOf(' '));
                if (temp <= System.Convert.ToInt32(sTempMin))
                { errOutputCpuTemperature = " PLEASE CHECK THE CPU TEMPERATURE! "; }

                // if the CPU temperature to high show a warning
                else
                {
                    string sTempMax = tbCpuTempMax.Text.Remove(tbCpuTempMax.Text.IndexOf(' '));
                    if (temp > System.Convert.ToInt32(sTempMax))
                    { errOutputCpuTemperature = " PLEASE CHECK THE CPU TEMPERATURE! "; }
                }
                tbStatusOutput.Text = errOutputCpuTemperature;
            }
            catch(Exception ex)
            {
                // show exception
                MessageBox.Show(ex.Message);
            }
        }

        // forward command to enable or disable the controls of the page
        void setPageIsEnabled(bool value)
        {
            try
            {
                setPageIsEnabledDelegate tempDelegate = new setPageIsEnabledDelegate(this.setEnabled);
                this.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Normal, tempDelegate, value);
            }
            catch { }
        }

        // enable or disable the controls of the page
        void setEnabled(bool value)
        {
            // if the client is disconnected make the recDisable visible
            if (value == false)
            {
                tbStatusOutput.Text = "NO CONNECTION";
                recDisable.Visibility = Visibility.Visible;
                recDisable_CPUTemp.Visibility = Visibility.Visible;
            }
            // if the client is connected make the recDisable invisible
            else
            {
                tbStatusOutput.Text = "CONNECTED";
                recDisable.Visibility = Visibility.Collapsed;
                recDisable_CPUTemp.Visibility = Visibility.Collapsed;
            }
        }
        #endregion
    }
} 
